<?php

namespace Drp\NativeEventApp\Templates\DataProviders;

use Drp\NativeEventApp\Exceptions\InvalidTemplate;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Templates\DataType;
use Illuminate\Support\Arr;

class DataProviderRegistry
{
    /** @var array */
    protected $resolvers = [];

    /**
     * @var \Drp\NativeEventApp\Test\Support\Models\Registration
     */
    protected $registration;

    public function __construct(array $resolvers = [])
    {
        $this->resolvers = $resolvers;
    }

    public function bind(string $tag, callable $resolver): self
    {
        $this->resolvers[$tag] = $resolver;

        return $this;
    }

    public function get(string $tag): ?DataProvider
    {
        return Arr::get($this->resolvers, $tag)($this->registration);
    }

    public function for(Registration $registration): self
    {
        $this->registration = $registration;

        return $this;
    }

    /**
     * @return mixed|object
     */
    public function resolve(string $key)
    {
        $parts = explode('.', $key);
        $providerName = array_shift($parts);
        $provider = $this->get($providerName);

        return array_reduce($parts, function ($carry, $resolve) use ($key) {
            if ($this->isFunction($resolve)) {
                $type = DataType::calculate($carry);

                $parts = $this->breakUpFunction($resolve);
                $name = $parts['name'];

                throw_unless($type->canHandleMethod($name), InvalidTemplate::badFunction($name, $key));

                return $carry->$name(...explode(', ', $parts['parameters']));
            }

            return $carry->$resolve;
        }, $provider);
    }

    private function isFunction(string $resolve): bool
    {
        return preg_match('/\(.+\)/', $resolve);
    }

    private function breakUpFunction(string $function): array
    {
        $matches = [];
        preg_match('/(?<name>[A-z]+)\((?<parameters>[A-z_.=\/, ]+)\)/', $function, $matches);

        return $matches;
    }
}
