<?php

namespace Drp\NativeEventApp\Test\Api\FormSubmissions;

use Drp\NativeEventApp\Forms\Facades\FormRegistry;
use Drp\NativeEventApp\Forms\Facades\FormSubmission;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Drp\NativeEventApp\Test\Support\Fakes\FakeForm;
use Drp\NativeEventApp\Test\Support\Fakes\FakeFormRegistry;
use Spatie\Snapshots\MatchesSnapshots;

class FormSubmissionTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function returns_correct_submission_data()
    {

        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
            'forename' => 'John',
            'surname' => 'Smith',
        ]);

        FormSubmission::handle('fake-form', FakeForm::class);

        $mutation = 'mutation {
            submitForm(event: 1, type: "fake-form", data: "{\"name\":\"test\",\"question\":\"test\"}") {
              status,
              message,
              submissionData
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest(
            $mutation
        ));
    }

    /** @test */
    public function returns_correct_validation_errors()
    {
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
            'forename' => 'John',
            'surname' => 'Smith',
        ]);

        FormSubmission::handle('fake-form', FakeForm::class);

        $mutation = 'mutation {
            submitForm(event: 1,type: "fake-form", data: "{\"question\":\"test\"}") {
              status,
              message,
              submissionData
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest(
            $mutation
        ));
    }

    /** @test */
    public function returns_correct_error_when_form_doesnt_exist()
    {
        $mutation = 'mutation {
            submitForm(event: 1, type: "fake-form", data: "{\"name\":\"test\",\"question\":\"test\"}") {
              status,
              message,
              submissionData
            }
        }';

        $this->assertMatchesJsonSnapshot($this->authenticatedGraphQlRequest(
            $mutation
        ));
    }
}
