<?php

namespace Drp\NativeEventApp\Test\Api;

use Carbon\Carbon;
use Drp\NativeEventApp\Test\Support\Builders\QuestionBuilder;
use Drp\NativeEventApp\Test\Support\Builders\QuizBuilder;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class QuizTest extends GraphQLTestCase
{
    use MatchesSnapshots, DatabaseMigrations;

    /** @var QuizBuilder */
    protected $quizBuilder;

    public function setUp(): void
    {
        parent::setUp();

        Carbon::setTestNow('01-01-2019');

        $this->quizBuilder = app(QuizBuilder::class);
    }

    /** @test */
    public function returns_quiz_questions_for_specific_quiz(): void
    {
        $this->registerFakeEvent();

        $this->quizBuilder
            ->forEvent($this->event)
            ->withQuestions(static function (QuestionBuilder $builder) {
                return $builder->withAnswers([
                    ['is_correct' => true, 'title' => 'First Answer', 'value' => 'first'],
                    ['is_correct' => false, 'title' => 'Second Answer', 'value' => 'second'],
                ])->makeMany([
                    ['title' => 'Quiz Question 1'],
                    ['title' => 'Quiz Question 2'],
                ]);
            })
            ->create();

        $query = '{
            events(id:1){
                id,
                quizzes {
                    id,
                    title,
                    type,
                    questions {
                        id,
                        title,
                        type,
                        show_correct,
                        answers {
                            id,
                            title,
                            value,
                            quizQuestion {
                                id,
                                title
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }
}
