<?php

namespace Drp\NativeEventApp\Test\Unit\Console\Commands;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\Event;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Notifications\Events\SendNotification;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Support\Facades\Event as EventFacade;

class SendScheduledNotificationsTest extends TestCase
{
    /** @test */
    public function it_emits_the_send_notification_event_for_unsent_notifications(): void
    {
        Carbon::setTestNow('02-04-2020 14:00');
        $event = factory(Event::class)->create();
        EventFacade::fake();

        // Should send
        Notification::make($event, 'Title', 'A')
            ->sendAt(now())
            ->publish();

        factory(Notification::class)->state('published')->create([
            'title' => 'Title',
            'message' => 'B',
            'send_at' => now()->subHour(),
        ]);

        // Should not send
        $notificationC = Notification::make($event, 'Title', 'C')
            ->sendAt(now()->addHour())
            ->publish();

        $this->artisan('notifications:send');

        EventFacade::assertDispatched(SendNotification::class, 2);

        EventFacade::assertNotDispatched(
            SendNotification::class,
            static function (SendNotification $event) use ($notificationC) {
                return $event->notification->id === $notificationC->id;
            }
        );
    }
}
